// Cube.cpp: implementation of the CCube class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Cube.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CBindTexture CCube::Textures;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCube::CCube(double r, double x, double y, double z, double rx, double ry, double rz, double vx, double vy, double vz, double vrx, double vry, double vrz, bool changeable, bool transparent, unsigned char alpha, CString bmp_texture_file)
{
	m_r = r/2;
	m_rx = rx;
	m_ry = ry;
	m_rz = rz;
	m_x = x;
	m_y = y;
	m_z = z;
	m_vrx = vrx;
	m_vry = vry;
	m_vrz = vrz;
	m_vx = vx;
	m_vy = vy;
	m_vz = vz;
	m_changeable = changeable;
	m_transparent = transparent;
	m_Texture_num = -1;

	if (bmp_texture_file != "")
		m_Texture_num = Textures.LoadTexture(bmp_texture_file, transparent, alpha);

	if (!m_changeable)
	{
		m_num_list = glGenLists(1);
		glNewList(m_num_list, GL_COMPILE);
			DrawCube();
		glEndList();
	}

}

CCube::~CCube()
{
	if (m_Texture_num != -1)
		Textures.FreeTexture(m_Texture_num);
	if (!m_changeable)
		glDeleteLists(m_Texture_num, 1);
}

void CCube::Draw(float x, float y, float z, bool multitexturing)
{
	glPushMatrix();	
		glTranslated(m_x + x, m_y + y, m_z + z);
		glRotated(m_rx, 1.0, 0.0, 0.0);
		glRotated(m_ry, 0.0, 1.0, 0.0);
		glRotated(m_rz, 0.0, 0.0, 1.0);
		if (multitexturing)
		{
			GLfloat amplitude = 1.0f;
			GLfloat color = 0.5f;

			glColor3f(amplitude, amplitude, amplitude);

			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glDepthMask(GL_TRUE);
			glDisable(GL_LIGHTING);
			glDisable(GL_DITHER);
			glShadeModel(GL_FLAT);
			glEnable(GL_TEXTURE_2D);
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);

			GLfloat ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			GLfloat diffuse[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			GLfloat specular[] = { 0.5f, 0.5f, 0.5f, 1.0f };
			GLfloat shininess  = 12.0;
			glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
			glMaterialf(GL_FRONT, GL_SHININESS, shininess);

			glEnable(GL_POLYGON_OFFSET_FILL);
			glPolygonOffset(-0.05f, -0.05f);			// ?
			glEnable(GL_LIGHTING);
			glDisable(GL_TEXTURE_2D);
			glDepthMask(GL_FALSE);
			glEnable(GL_DITHER);
			glBlendFunc(GL_ONE, GL_ONE);
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);
			glDepthFunc(GL_LEQUAL);
			glDepthMask(GL_TRUE);
			glDisable(GL_POLYGON_OFFSET_FILL);
			glDisable(GL_BLEND);
			glEnable(GL_TEXTURE_2D);
		}
		else
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);
	glPopMatrix();
}

void CCube::Update(int t)
{
	m_x += m_vx * t;
	m_y += m_vy * t;
	m_z += m_vz * t;

	m_rx += m_vrx * t;
	m_ry += m_vry * t;
	m_rz += m_vrz * t;

}

void CCube::DrawCube()
{
	if (m_Texture_num != -1)
		glBindTexture(GL_TEXTURE_2D, m_Texture_num);

    glBegin(GL_QUADS);
		glNormal3d(0.0,0.0,1.0);
		glTexCoord2f(0.0, 1.0); glVertex3d(-m_r,  m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d(-m_r, -m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, -m_r, m_r);
		glTexCoord2f(0.0, 0.0); glVertex3d( m_r,  m_r, m_r);

		glNormal3d(0.0,0.0,-1.0);
		glTexCoord2f(0.0, 1.0); glVertex3d(  m_r,  m_r, -m_r);
		glTexCoord2f(0.0, 0.0); glVertex3d(  m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( -m_r,  m_r, -m_r);

		glNormal3d(1.0,0.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, -m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( m_r, m_r, -m_r);

		glNormal3d(-1.0,0.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( -m_r, -m_r, m_r);

		glNormal3d(0.0,-1.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, -m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( m_r, -m_r, m_r);

		glNormal3d(0.0,1.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( -m_r, m_r, -m_r);
	glEnd();
}

