// Joueur.cpp: implementation of the CJoueur class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Kurushi.h"
#include "Joueur.h"
#include "Math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define PI 3.14159265f

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CJoueur::CJoueur(char *filename)
{
	m_x = 0;
	m_y = 0;
	m_z = 0;
	m_angle = 0;
	m_num_list = -1;
	m_g=m_d=m_b=m_h=m_a1=m_a2 = false;
	m_v = 0.005f;
	m_rv = 0.1f;
	m_ClipSol = true;
	m_ClipRC = true;
	max_pose = 1;
	pose = 0;
	if (filename)
	{
		m_object.glmReadOBJ(filename);
		m_object.glmUnitize(0.9f);
		m_object.glmFacetNormals();
		m_object.glmVertexNormals(90);
		m_object.glmSpheremapTexture();
		m_num_list = m_object.glmList(GLM_SMOOTH | GLM_TEXTURE);
		m_object.glmDimensions(m_dim);
		m_x = m_dim[0]/2;
		m_y = m_dim[1];
		m_z = -m_dim[2]/2;
	}
}

CJoueur::~CJoueur()
{
	if (m_num_list != -1)
		glDeleteLists(m_num_list, 1);
}

void CJoueur::Update(int t, CSol *s, CRollingCube *r)
{
	if (!s)
		return;
	if (!r)
		return;

	if (m_g)
		m_angle += m_rv * t;
	else if(m_d)
		m_angle -= m_rv * t;
	if (m_angle > 360)
		m_angle -=360;
	if (m_angle < 0)
		m_angle += 360;

	if (m_h)
	{
		m_x += m_v * t * float(cos(m_angle*PI/180 + PI/2));
		if (CheckCollision(s, r))
			m_x -= m_v * t * float(cos(m_angle*PI/180 + PI/2));
		m_z -= m_v * t * float(sin(m_angle*PI/180 + PI/2));
		if (CheckCollision(s, r))
			m_z += m_v * t * float(sin(m_angle*PI/180 + PI/2));
	} else if (m_b)
	{
		m_x -= m_v * t * float(cos(m_angle*PI/180 + PI/2));
		if (CheckCollision(s, r))
			m_x += m_v * t * float(cos(m_angle*PI/180 + PI/2));
		m_z += m_v * t * float(sin(m_angle*PI/180 + PI/2));
		if (CheckCollision(s, r))
			m_z -= m_v * t * float(sin(m_angle*PI/180 + PI/2));
	}

	if (m_a1 && pose < max_pose)
	{
		s->Marque(m_x, m_z, 0x00ff00);
		pose++;
	}

	if (m_a2)
	{
		s->Action(this, r);
		pose = 0;
	}
}

void CJoueur::Draw()
{
//	glDisable(GL_TEXTURE_2D);
	if (m_num_list != -1)
	{
		glPushMatrix();
			glTranslatef(m_x, m_y, m_z);
			glRotatef(m_angle, 0, 1, 0);
			glCallList(m_num_list);
		glPopMatrix();
	}
//	glEnable(GL_TEXTURE_2D);
}


bool CJoueur::CheckCollision(CSol * s, CRollingCube * r)
{
	if (m_ClipSol)
		if (m_x - m_dim[0]/2 < 0 || m_x + m_dim[0]/2 > s->GetSizex()*s->m_r || m_z + m_dim[2]/2 > 0 || m_z - m_dim[2]/2 < -s->GetSizez()*s->m_r)
			return true;
	if (m_ClipRC)
		if (r->Cube(m_x, m_z))
			return true;
	return false;
}
